function [compensated,cell_ris] = MEPI_code(gof,step)
% [compens_mobile,cell_ris] = mepi(gof,step)
% Estimate the motion vector of a sequence using the first frequency of the
% fourier transform 
%
% INPUT PARAMETERS
%   gof       = size of the Group Of Frame 
%   step      = block_size  (for example step = 16 for block size 16x16)
%
%
% OUTPUT PARAMETERS
%
%   compensated = it is a cell. Each element of the cell contains a
%   frame of the motion compensated sequence
%
%   cell_ris = it is a cell. Each element of the cell refers to one frame
%   of the sequence and contains a matrix: each row of the matrix contains
%   the coordinates of the topleft and bottomleft points of each analysed block of the frame, 
%   the motion vector and the value of the psnr of the motion compensated block with respect to 
%   the previous frame. The variable is also stored in the file
%   mepidata.mat
%
%
%
%
%---------- load the sequence that has been stored in a matfile
% the function load a file .mat that contains the sequence: the file
% consists of a cell variable c. Each element of the cell contains a frame
% of the sequence. Hemce the number of frames (no_frames) is the length of
% the variable c


 
   load mobile.mat 
% load flower.mat
 %load coastguard.mat 

no_frames = length(c);

%---------------------------------------------------------------
%  eliminate eventual zeros
%---------------------------------------------------------------

v = c;
  for i = 1:no_frames
      app=v{i};
      v{i} = app(2:end,3:end);
  end
c = v;

%---------------------------------------------------------------
  
PSNR = 0;
MAD = 0; 
mad=0;
cell_ris = cell(1,no_frames-1); 
tic


% process all the sequence
for frame = 1:no_frames-(gof-1)
    
    vett = v{frame};
  
    finerow = size(vett,1);
    finecol = size(vett,2);
    risultato = [];
    
    for col = 1:step:finecol-step
        cdim1 = col;
        cdim2 = col+step-1;
        for row = 1:step:finerow-step
            vett = v{frame};            
            rdim1 = row;
            rdim2 = row + step-1;
            vett = vett(rdim1:rdim2,cdim1:cdim2);
            temp = zeros(size(vett));
            
            for i=0:gof-1  
                tempor = v{i+frame};               
                tempor = tempor(rdim1:rdim2,cdim1:cdim2);
             
             % compute the sum of subsequent blocks
                temp = temp + tempor;
            end
           
            
            % vett is now the first frame of the considered gof 
            vett = v{frame};
            vett = vett(rdim1:rdim2,cdim1:cdim2);
            vetto = [vett];
            tempnew = temp;
            
            % fft computation
            sec = fft2(vetto);
            G_t = fft2(temp);
            
            warning off MATLAB:divideByZero
            G = fft2(tempnew).* conj(sec) ./ abs(sec).^2;
            G_t = fft2(temp);
                      
            %motion computation
           
            omega = 2;
            A = angle(G);
      
            stima_dx = -(size(G,1))*A(omega,1)/((gof-1)*pi*(omega-1));
            stima_dy = -(size(G,2))*A(1,omega)/((gof-1)*pi*(omega-1));
            stima_dx = mod(stima_dx,step/2*sign(stima_dx));
            stima_dy = mod(stima_dy,step/2*sign(stima_dy));

            K = gof;
            
            
          
            % error estimation
                    
          
           bo = .9997;
           std_noiseRE = real(G_t)-(1+bo)*real(sec)+imag(sec)*sqrt(1-bo^2);
           std_noiseIM = imag(G_t)-(1+bo)*imag(sec)-real(sec)*sqrt(1-bo^2);
           C = (length(G_t))/((K-1)*pi*(omega-1));
           Errorex = abs(sqrt((-1.4*imag(G_t(omega,1))/abs(G_t(omega,1))^2 *std_noiseRE(omega,1))^2 +(1.4*real(G_t(omega,1))/abs(G_t(omega,1))^2 *std_noiseIM(omega,1))^2  + .5* (imag(G_t(omega,1))^2-real(G_t(omega,1))^2)/abs(G_t(omega,1))^4 *std_noiseRE(omega,1)*std_noiseIM(omega,1) -0*0.25*real(G_t(omega,1))*imag(G_t(omega,1))/(abs(G_t(omega,1))^4) * (std_noiseRE(omega,1)^2-std_noiseIM(omega,1)^2)))*C/2;
           Errorey = abs(sqrt((-1.4*imag(G_t(1,omega))/abs(G_t(1,omega))^2 *std_noiseRE(1,omega))^2 +(1.4*real(G_t(1,omega))/abs(G_t(1,omega))^2 *std_noiseIM(1,omega))^2  + .5* (imag(G_t(1,omega))^2-real(G_t(1,omega))^2)/abs(G_t(1,omega))^4 *std_noiseRE(omega,1)*std_noiseIM(omega,1) -0*0.25*real(G_t(1,omega))*imag(G_t(1,omega))/(abs(G_t(1,omega))^4) * (std_noiseRE(1,omega)^2-std_noiseIM(1,omega)^2)))*C/2;
           
           Mx = [];
           My = [];
           ps = [];
           stima_dxp=stima_dx;
           stima_dyp=stima_dy;
           for j = 1:16
               if j==1 
                   stima_dx = round(mod((stima_dxp),step/2*sign(stima_dxp)));
                   stima_dy = round(mod((stima_dyp),step/2*sign(stima_dyp)) );
               end
               if j==2 
                   stima_dx = round(mod((stima_dxp),step/2*sign(stima_dxp)) + Errorex);
                   stima_dy = round(mod((stima_dyp),step/2*sign(stima_dyp)) + Errorey);
               end
               % %        
               
               if j==3 
                   stima_dx = round(mod((stima_dxp),step/2*sign(stima_dxp)) - Errorex);
                   stima_dy = round(mod((stima_dyp),step/2*sign(stima_dyp)) + Errorey);
               end
               
               if j==4 
                   stima_dx = round(mod((stima_dxp),step/2*sign(stima_dxp)) + Errorex);
                   stima_dy = round(mod((stima_dyp),step/2*sign(stima_dyp)) - Errorey);
               end
               
               if j==5 
                   stima_dx = round(mod((stima_dxp),step/2*sign(stima_dxp)) - Errorex);
                   stima_dy = round(mod((stima_dyp),step/2*sign(stima_dyp)) - Errorey);
               end
               
               if j==6 
                   stima_dx =round(mod((stima_dxp),step/2*sign(stima_dxp))) ;
                   stima_dy = round(mod((stima_dyp),step/2*sign(stima_dyp)) + Errorey);
               end
               
               if j==7 
                   stima_dx = round(mod((stima_dxp),step/2*sign(stima_dxp))) ;
                   stima_dy = round(mod((stima_dyp),step/2*sign(stima_dyp)) - Errorey);
               end
               
               if j==8 
                   stima_dx = round(mod((stima_dxp),step/2*sign(stima_dxp)) + Errorex);
                   stima_dy = round(mod((stima_dyp),step/2*sign(stima_dyp)) );
               end
               if j==9 
                   stima_dx = round(mod((stima_dxp),step/2*sign(stima_dxp)) - Errorex);
                   stima_dy =  round(mod((stima_dyp),step/2*sign(stima_dyp)) );
               end
               
               
               if j==10 
                   stima_dx = round(mod((stima_dxp),step/2*sign(stima_dxp))) ;
                   stima_dy = round(mod((stima_dyp),step/2*sign(stima_dyp)) + 2*Errorey);
               end
               
               if j==11 
                   stima_dx = round(mod((stima_dxp),step/2*sign(stima_dxp))) ;
                   stima_dy = round(mod((stima_dyp),step/2*sign(stima_dyp)) - 2*Errorey);
               end
               
               if j==12 
                   stima_dx = round(mod((stima_dxp),step/2*sign(stima_dxp)) + 2*Errorex);
                   stima_dy = round(mod((stima_dyp),step/2*sign(stima_dyp)) );
               end
               if j==13 
                   stima_dx = round(mod((stima_dxp),step/2*sign(stima_dxp)) - 2*Errorex);
                   stima_dy = round(mod((stima_dyp),step/2*sign(stima_dyp)) ) ;
               end
%                if j==14 
%                    stima_dx = -round(mod((stima_dxp),step/2*sign(stima_dxp)) ) ;
%                    stima_dy = round(mod((stima_dyp),step/2*sign(stima_dyp)) );
%                end
%                
%                if j==15 
%                    stima_dx = round(mod((stima_dxp),step/2*sign(stima_dxp)) ) ;
%                    stima_dy = -round(mod((stima_dyp),step/2*sign(stima_dyp)) );
%                end
%                
%                if j==16 
%                    stima_dx = -round(mod((stima_dxp),step/2*sign(stima_dxp)));
%                    stima_dy = -round(mod((stima_dyp),step/2*sign(stima_dyp)) );
%                end

               
               stima_dx=round(stima_dx);
               stima_dy=round(stima_dy);
               
               
               if (abs(stima_dx)>=step/2) | (size(vetto,1)<step)%4
                   stima_dx = 0;
               end            
               if abs(stima_dy)>=step/2  | (size(vetto,2)<step)%4  
                   stima_dy = 0;
               end
               
              
               % PSNR evaluation 
               
               vett = c{frame};
               vett = vett(rdim1:rdim2,cdim1:cdim2);
               
               new = c{frame+1};
               new = new(rdim1:rdim2,cdim1:cdim2);
               
               
               if (stima_dx>=0) & (stima_dy>=0)
                   
                   refer = vett(1:end-stima_dx,1:end-stima_dy);
                   newnew = zeros(size(refer));
                   newnew = new(stima_dx+1:end,stima_dy+1:end);
               end
               
               if (stima_dx<0) & (stima_dy>=0)
                   
                   refer = vett(abs(stima_dx)+1:end,1:end-stima_dy);
                   newnew = zeros(size(refer));
                   newnew = new(1:end-abs(stima_dx),stima_dy+1:end);
               end
               
               if (stima_dx>=0) & (stima_dy<0)
                   
                   refer = vett(1:end-stima_dx,abs(stima_dy)+1:end);
                   newnew = zeros(size(refer));
                   newnew = new(stima_dx+1:end,1:end-abs(stima_dy));
               end
               
               if (stima_dx<0) & (stima_dy<0)
                   
                   refer = vett(abs(stima_dx)+1:end,abs(stima_dy)+1:end);
                   newnew = zeros(size(refer));
                   newnew = new(1:end-abs(stima_dx),1:end-abs(stima_dy));
               end
      
               
               mse = mean2((refer-newnew).^2);
               psnr = 10*log10(255^2/(mse));
               if psnr==Inf | psnr == nan
                   psnr=300;
               end
               ps = [ps psnr];
               Mx = [Mx stima_dx];
               My = [My stima_dy];
           end
           [mm pm] = max(ps);
           stima_dx =Mx(pm(1));
           stima_dy =My(pm(1));
           psnr = ps(pm(1));

           risultato = [risultato; rdim1 rdim2 cdim1 cdim2 stima_dx stima_dy psnr];
        end
    end
    cell_ris{frame} = risultato; 
    PSNR = PSNR + mean(risultato(:,7)); 

end

toc


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
save mepidata.mat cell_ris


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%motion compensation
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


v = c;
  for i = 1:no_frames
      app=v{i};
      v{i} = app(2:end,3:end);
  end
c = v;

I = c{1};
original{1} =c{1}(step+1:size(I,1)-step,step+1:size(I,2)-step);
compensated{1} =c{1}(step+1:size(I,1)-step,step+1:size(I,2)-step);


for nume = 1:no_frames-1%

I = c{nume};
J = c{nume+1};

ris = cell_ris{nume};
brow = step+1;
brow2 = size(I,1)-step;
bcol = step+1;
bcol2 = size(I,2)-step;


MAT = [];
for i = 1:size(ris,1)
    MAT(ris(i,1):ris(i,2)-step/2,ris(i,3):ris(i,4)) = -ris(i,5);
    MAT(ris(i,2)-(step/2-1):ris(i,2),ris(i,3):ris(i,4)) = -ris(i,6);
    
end

H = I;

for row = brow:step:brow2-step+1
    for col = bcol:step:bcol2-step+1
        stima_dx = MAT(row,col);
        stima_dy = MAT(row+step-1,col);    
           if (stima_dx>=0) & (stima_dy>=0)
                H(row:row+step-1,col:col+step-1)= I(stima_dx+row:row+step-1+stima_dx,stima_dy+col:col+step-1+stima_dy);
           end             
           if (stima_dx<0) & (stima_dy>=0)
                H(row:row+step-1,col:col+step-1) = I(row-abs(stima_dx):row+step-1-abs(stima_dx),stima_dy+col:col+step-1+stima_dy);
           end            
           if (stima_dx>=0) & (stima_dy<0)
                H(row:row+step-1,col:col+step-1) = I(stima_dx+row:row+step-1+stima_dx,col-abs(stima_dy):col+step-1-abs(stima_dy));
           end            
           if (stima_dx<0) & (stima_dy<0)
                H(row:row+step-1,col:col+step-1) = I(row-abs(stima_dx):row+step-1-abs(stima_dx),col-abs(stima_dy):col+step-1-abs(stima_dy));
           end            
    end
end


subplot(1,2,1), imshow(H(brow:brow2,bcol:bcol2),[])
title('motion compensated')
subplot(1,2,2), imshow(J(brow:brow2,bcol:bcol2),[]), title('original')
pause(.2)



original{nume+1} = J(brow:brow2,bcol:bcol2);
compensated{nume+1} = H(brow:brow2,bcol:bcol2);
end

